/* $Id: e_mod_main.c 167 2008-07-13 20:53:32Z tamentis $ */

/*
 * Copyright (c) 2008 Bertrand Janin <tamentis@neopulsar.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include <e.h>
#include <e_intl.h>
#include <stdio.h>
#include <time.h>


static E_Module *tebsclock_module = NULL;


typedef struct _Instance {
	Ecore_Timer *timer;
	Evas_Object *clock;
	E_Gadcon_Client *gcc;
	int cur_s;
	int cur_m;
	int cur_h;
} Instance;


/* woot() - Updates the Edje images status (tick every second). */
int
woot(Instance *inst, Evas_Object *o, const char *em, const char *source)
{
	time_t now;
	struct tm *tm;
	int i;
	char signame[20] = "nothing";

	now = time(NULL);
	tm = localtime(&now);

	/* Update Hours. */
	if (tm->tm_hour != inst->cur_h) {
		for (i = 0; i < 6; i++) {
			sprintf(signame, "%s_hour_%u",
				(tm->tm_hour & 1<<i ? "set" : "unset"), i);
			if ((tm->tm_hour & 1<<i) != (inst->cur_h & 1<<i)) {
				edje_object_signal_emit(inst->clock, signame, "");
			}
		}
		inst->cur_h = tm->tm_hour;
	}

	/* Update Minutes. */
	if (tm->tm_min != inst->cur_m) {
		for (i = 0; i < 6; i++) {
			sprintf(signame, "%s_minute_%u",
					(tm->tm_min & 1<<i ? "set" : "unset"), i);
			if ((tm->tm_min & 1<<i) != (inst->cur_m & 1<<i)) {
				edje_object_signal_emit(inst->clock, signame, "");
			}
		}
		inst->cur_m = tm->tm_min;
	}

	/* Update Seconds. */
	for (i = 0; i < 6; i++) {
		sprintf(signame, "%s_second_%u",
				(tm->tm_sec & 1<<i ? "set" : "unset"), i);
		if ((tm->tm_sec & 1<<i) != (inst->cur_s & 1<<i))
			edje_object_signal_emit(inst->clock, signame, "");
	}

	inst->cur_s = tm->tm_sec;

	return 1;
}


/* gadcon requirements */
static E_Gadcon_Client *_gc_init(E_Gadcon *gc, const char *name, const char *id, const char *style);
static void _gc_shutdown(E_Gadcon_Client *gcc);
static void _gc_orient(E_Gadcon_Client *gcc);
static char *_gc_label(void);
static Evas_Object *_gc_icon(Evas *evas);
static const char *_gc_id_new(void);
/* and actually define the gadcon class that this module provides (just 1) */
static const E_Gadcon_Client_Class _gadcon_class =
{
	GADCON_CLIENT_CLASS_VERSION,
	  "tebsclock",
	  {
		  _gc_init, _gc_shutdown, _gc_orient, _gc_label, 
		  _gc_icon, _gc_id_new, NULL
	  },
	E_GADCON_CLIENT_STYLE_INSET
};


/* _gc_icon() - Pull the icon (for menus, etc.) */
static Evas_Object *
_gc_icon(Evas *evas)
{
	Evas_Object *o;
	char buf[4096];

	o = edje_object_add(evas);
	snprintf(buf, sizeof(buf), "%s/e-module-tebsclock.edj",
			e_module_dir_get(tebsclock_module));
	edje_object_file_set(o, buf, "icon");
	return o;
}



static E_Gadcon_Client *
_gc_init(E_Gadcon *gc, const char *name, const char *id, const char *style)
{
	Evas_Object *o;
	E_Gadcon_Client *gcc;
	Instance *inst;
	char buf[4096];

	inst = E_NEW(Instance, 1);

	o = edje_object_add(gc->evas);
	e_theme_edje_object_set(o, "base/theme/modules/tebsclock",
			"e/modules/tebsclock/main");

	snprintf(buf, sizeof(buf), "%s/e-module-tebsclock.edj",
			e_module_dir_get(tebsclock_module));
	edje_object_file_set(o, buf, "test");

	inst->cur_s = 0;
	inst->cur_m = 0;
	inst->cur_h = 0;

	inst->timer = ecore_timer_add(1, woot, inst);

	gcc = e_gadcon_client_new(gc, name, id, style, o);
	gcc->data = inst;

	inst->clock = o;
	inst->gcc = gcc;

	e_gadcon_client_util_menu_attach(gcc);

	return gcc;
}

static void
_gc_shutdown(E_Gadcon_Client *gcc)
{
	Instance *inst;

	inst = gcc->data;
	evas_object_del(inst->clock);
	ecore_timer_del(inst->timer);
	E_FREE(inst);
}

static void
_gc_orient(E_Gadcon_Client *gcc)
{
	e_gadcon_client_aspect_set(gcc, 16, 16);
	e_gadcon_client_min_size_set(gcc, 16, 16);
}

static char 
*_gc_label(void)
{
	return gettext("TebsClock");
}

static const char *
_gc_id_new(void)
{
	return _gadcon_class.name;
}


/* module setup */
EAPI E_Module_Api e_modapi =
{
	E_MODULE_API_VERSION,
	"tebsclock"
};

EAPI void *
e_modapi_init(E_Module *m)
{
	tebsclock_module = m;
	      
	e_gadcon_provider_register(&_gadcon_class);
	return m;
}

EAPI int
e_modapi_shutdown(E_Module *m)
{
	tebsclock_module = NULL;

	e_gadcon_provider_unregister(&_gadcon_class);
	return 1;
}

EAPI int
e_modapi_save(E_Module *m)
{
	return 1;
}


